;**********************************************************************
;    Filename:     servo-weiche.asm                                   *
;    Date:         22.08.2016                                         *
;    File Version: 170703                                                   *
;                                                                     *
;    Author:                                                          *
;    Company:           C&S Gbel                                     *
;    Version:           160821-v10                                    *
;    mit Leiterplatte:  LP160821-v9  LP160821-v10                     *
;    Letzte nderung :  03.07.2017                                    *
;   ----------------------------------------------------------------- *
;    Prozessor PIC12F683 Beschaltung des IC                           
;                            _____                                    *
;    GP2 IMP              A |     | E GP3  Taster Programmiermodus    *
;    GP1 Poti/Rckmld AN1/A |     | A GP4  LED Anzeige Programmiermod *
;    GP0 rechts           E |     | E GP5  links                      *
;    GND                  - |____X| + +5V                             *
;                                                                     *
;   ----------------------------------------------------------------- *
;      Zeitkonstanten fr 4MHz Nutzung des Timer0 mit INTERRUPT       *
;       Zeit        Vorteiler   TMR0                                  *
;       18,0ms      1:128       115     konstanter Teil               *
;        1,0ms      1:8         131     var. Teil linker Anschlag     *
;        1,5ms      1:8          68     var. Teil Mittelstellung      *
;        2,0ms      1:8           7     var. Teil rechter Anschlag    *
;   variabler + konstanter Teil 19,0 bis 20ms -- dieser Wert          *
;   entspricht in etwa der empfolenen Zycluszeit                      *
;   ----------------------------------------------------------------- *
;   Zeitkonstanten fr blinken der LED mit Timer1                     *
;       TMR1L = 0x00                                                  *
;       TMR1H = 0x3E                                                  *
;       Vorteiler   T1CKPS1:0                                         *
;                       11      1:8 langer Impuls                     *
;                       10      1:4 kurzer Impuls                     *
;                       01      1:2 pause                             *
;**********************************************************************
;                                                                     *
;       Files Required: P12F683.INC                                   *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************
    list      p=12F683        ; list directive to define processor
    #include <p12F683.inc>    ; processor specific variable definitions
    errorlevel  -302          ; suppress message 302 from list file
; _MCLRE_OFF um das Pin 4 als Eingang zu nutzen
; _INTRC_OSC_NOCLKOUT Nutzung des internen Generators mit 4MHz
; 
    __CONFIG   _FCMEN_ON & _IESO_OFF & _CP_OFF & _CPD_OFF & _BOD_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT 
; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.
; ------------------------------------------------------------------------------------
; Daten im EEPROM
    org H'2100'
    de "(c) C&S Goebel 03.07.2017 mit LP160821-v10", 0
    org H'2170'
    de 0x84                 ; Startwert fr min Linkslage bei 1. Start -- danach Einstellwert
    org H'2171'
    de 0xC0                 ; Startwert fr max Rechtslage bei 1. Start -- danach Einstellwert
    org H'2160'
    de 0xC0                 ; Startwert fr letzte Einstellung
    org H'2161'
    de 0x00                 ; Startwert fr var_fakt Drehgeschwindigkeit  bei 1. Start -- danach Einstellwert
    org H'2172'
    de 0xFD
;***** VARIABLE DEFINITIONS
z_entprell  EQU     0x72    ; Zhler fr die Tastenentprellung
ee_dat      EQU     0x73    ; Zwischenspeicher fr EEPROM-Programmierung 
var_zahl    EQU     0x74    ; Countdownzhler fr Stellgeschwindigkeit
var_time    EQU     0x75    ; variabler Impuls
long_time   EQU     0x76    ; Vorteiler 64* (255-180)=18ms Eintrag 75
x_stat      EQU     0x77    ; Statusbyte fr Abarbeitung
                            ; Bit0   =  Toggle Bit Timer0 var + konst = ca.20ms
                            ; Bit1   =  Statusbits fr Betriebsmodus
                            ;        =  1 Tastendruck erkannt
                            ;        =  0 Taste nicht oder zu kurz gedrckt (Prellzeit)
                            ; Bit2   =  Togglebit fr LED Blinkrythmus (ISR)
                            ;           TMR1H 0x03E TMR1L 0x000
                            ;           Vorteiler lang 1:8, kurz 1:4, Pause 1:2
                            ; Bit4:3 =  00 Einsprung in Anschlge Einstellen
                            ;             
                            ;        =  01 rechten Anschlag einstellen
                            ;        =  10 linken Anschlag einstellen
                            ; Bit5   =  1 Messung an GP1 (Analogmodus)
                            ;        =  0 Bearbeitung (Analogmodus)
                            ; Bit6   =  1 Analogmodus
                            ;        =  0 Digitalmodus
                            ; Bit7   = 
var_fakt    EQU     0x78    ; Faktor fr die Stellgeschwindigkeit
var_tm_ist  EQU     0x79    ; momentane Stellung des Servos
var_tm_soll EQU     0x7A    ; anzufahrende Stellung des Servos
li_lage     EQU     0x7B    ; linkslage
re_lage     EQU     0x7C    ; rechtslage
ana_pu      EQU     0x7D    ; Zwischenspeicher
w_temp      EQU     0x7E    ; WREG sichern fr ISR 
status_temp EQU     0x7F    ; STATUS sichern fr ISR
; *************** START **************************************************
    ORG     0x000           ; processor reset vector
    goto    main            ; go to beginning of program
; =============== ISR ====================================================
    ORG     0x004           ; interrupt vector location
    movwf   w_temp          ; save off current W register contents
    swapf   STATUS,W        ; move status register into W register
    movwf   status_temp     ; save off contents of STATUS register
    clrf    STATUS          ; Bank0 und alle Flags 0
; ------------- Verzweigung PWM zur Blinkfunktion ------------------------
    btfss   INTCON,T0IF     ; Test ob Timer0-Interrupt
    goto    int_led         ; nein also Timer1-Interrupt
; -------------- PWM -----------------------------------------------------
int_pwm
    btfss   x_stat,0        ; wenn 1 dann 18ms          
    goto    int_pwm_var     ; springe zu variablen Puls 
; ---- PWM statischer Teil 18ms ------------------------------------------
    movlw   D'115'          ; 18ms
    movwf   TMR0            ; Timer0 laden
    BSF     STATUS,RP0      ; Bank1
    bcf     OPTION_REG,PS0  ; Timer0 Vorteiler 1:128
    bsf     OPTION_REG,PS1  
    bsf     OPTION_REG,PS2
    BCF     STATUS,RP0      ; Bank0
    ; Behandlung der Wartezeit fr die Entprellung der Tasten
    movf    z_entprell,F    ; testen auf = 0 
    btfss   STATUS,Z        ; wenn Z-Flag = 1 berspringen decf
    decf    z_entprell,F
    ; Entprell Zhler Ende
    bcf     GPIO,GP2        ; Impuls aus
    bcf     x_stat,0        ; Toggle - wird im variablen Teil (1-2ms) eingeschaltet
; ---- Behandlung Analogmodus  Hier alle 19 - 20ms Messung starten -------
    btfsc   ADCON0,GO       ; Test ob Messung beendet 
    goto    int_pwm_end     ; Messung luft noch
    btfsc   x_stat,6        ; abfragen Analogmodus
    bsf     ADCON0,GO       ; Messung neu starten
    goto    int_pwm_end
; -----------Behandlung variabler Impuls 1ms - 2ms -----------------------
int_pwm_var
    movf    var_tm_ist,W
    xorwf   var_tm_soll,W   ; wenn beide gleich << dann Z-Flag = 1
    btfsc   STATUS,Z        ; die Werte sind ungleich
    goto    int_pwm_ree     ; Endwert erreicht - var_tm_ist = var_tm_soll
; var_fakt testen auf 0
    movf    var_fakt,F      ; in sich selbst laden << wenn 0 ist Z-Flag = 1
    btfsc   STATUS,Z
    goto    int_pwm_sp1     ; schnelle Verstellung
    movf    var_zahl,F      ; Zhler auf 0 testen << wenn 0 ist Z-Flag = 1
    btfss   STATUS,Z
    goto    int_pwm_sp2
    movf    var_fakt,W      ; Den zhler neu laden
    movwf   var_zahl
int_pwm_sp3
    movf    var_fakt,W
    movwf   var_zahl
    movf    var_tm_ist,W    ; !! subwf es wird das WREG vom F-Register abgezogen
    subwf   var_tm_soll,W   ; wenn F-Register kleiner als das XREG ist
    btfsc   STATUS,C        ; ist das C-Bit = 0 ist negative Zahl
    goto    int_pwm_radd
int_pwm_rsub
    decf    var_tm_ist,F    ; IST-Wert verringern
    goto    int_pwm_ree
int_pwm_radd    
    incf    var_tm_ist,F    ; IST-Wert erhhen
int_pwm_ree                 
    movf    var_tm_ist,W    ; momentaner Wert IST-Stellung
    movwf   var_time        ; in var_time laden
; var_time in der Timer0 laden und Vorteiler einstellen
    movf    var_time,W      ; Variabler Impuls aus Ablage
    movwf   TMR0            ; Timer0 laden
    BSF     STATUS,RP0      ; Bank1
    bsf     OPTION_REG,PS0  ; Timer0 Vorteiler 1:16
    bsf     OPTION_REG,PS1  
    bcf     OPTION_REG,PS2
    BCF     STATUS,RP0      ; Bank0
    bsf     GPIO,GP2        ; Impuls ein
    bsf     x_stat,0        ; Toggle << nchster Interrupt 18ms
int_pwm_end
; ------------------------
; Einschub Ermittlung Rechts- oder Linkslage durch Vergleich 
; li_lage mit var_tm_ist bzw. re_lage mit var_tm_ist
; wenn gleich dann ist Endlage erreicht und GP1 wird geschaltet
    movf    li_lage,W       ; Endwert links nach W
    subwf   var_tm_ist,W    ; Ist-Wert subtrahieren
    btfsc   STATUS,Z        ; nicht gleich re_lage testen   
    bcf     GPIO,GP1        ; Rueckmeldung = 0 an GP1
    movf    re_lage,w       ; Endwert rechts nach w
    subwf   var_tm_ist,w    ; Ist-Wert subtrahieren
    btfsc   STATUS,Z        ; nicht gleich also Ende
    bsf     GPIO,GP1        ; Rueckmeldung = 0 an GP1
; ------------------------
    bcf     INTCON,T0IF     ; Timer0 Interrupt zurcksetzen
; ---------- ISR Endebehandlung ------------------------------------------
int_end
    swapf   status_temp,W   ; retrieve copy of STATUS register
    movwf   STATUS          ; restore pre-isr STATUS register contents
    swapf   w_temp,F
    swapf   w_temp,W        ; restore pre-isr W register contents
    retfie                  ; return from interrupt
; --------- Bearbeitung schnelle Verstellung -----------------------------
int_pwm_sp1
    movf    var_tm_soll,W
    movwf   var_tm_ist
    goto    int_pwm_ree
int_pwm_sp2
    decf    var_zahl,F
    goto    int_pwm_ree
; ---------- LED Blinkbehandlung -------------------------------------
int_led
    btfss   PIR1,TMR1IF     ; Test ob Timer1 Interrupt gesetzt
    goto    int_bl_aus      ; kein Interrupt
; --------- test auf x_stat Bit4:3 auf 00
    movf    x_stat,W        ; x_stat in W 
    andlw   B'00011000'     ; Maske auf WREG wenn 00 dann Z-Flag
    btfsc   STATUS,Z        ; Test
    goto    int_bl_an       ; es war 00 also LED an
; --------- test auf x_stat Bit4:3 auf 11
    movf    x_stat,W
    xorlw   B'00011000'     ; Maske auf WREG auf 00 wurde vorher getestet
    btfsc   STATUS,Z        ; es war 11 also LED aus
    goto    int_bl_aus      ; 
; --------- blinken einschalten Zeiteinstellung in ta_prog ---------------
    btfsc   x_stat,2        ; wenn 1 >> ausschalten
    goto    int_bl_aus
int_bl_an
    bcf     GPIO,GP4        ; LED ein
    bsf     x_stat,2        ; Togglebit EIN
    goto    int_bl_end
int_bl_aus
    bsf     GPIO,GP4        ; LED aus
    bcf     x_stat,2        ; Togglebit AUS
int_bl_end
    bcf     PIR1,TMR1IF     ; Timer1-Intrrupt zurcksetzen
    goto    int_end
; =====================================================================
main
    nop
; ---------------------------------------------------------------------
; --- Anfangsinitialisierung 4 MHz IRCF2:0 ((110) ---------------------
init
    clrf    x_stat          ; Statusbits lschen
; Timer0 fr konstanten und variablen Impuls vorbereiten TMR0
    movlw   D'115'          ; fester Impuls 18ms mit Vorteiler 128 (110)
    movwf   long_time       ; Wert ablegen
    bsf     STATUS,RP0      ; Bank1
; li_lage wiederherstellen
    movlw   0x70            ; Adresse gespeicherte li_lage im EEPROM
    movwf   EEADR
    bsf     EECON1,RD       ; lesen
    movf    EEDAT,W         ; laden
    movwf   li_lage
; re_lage wiederherstellen
    movlw   0x71            ; Adresse gespeicherte re_lage im EEPROM
    movwf   EEADR
    bsf     EECON1,RD       ; lesen
    movf    EEDAT,W         ; laden
    movwf   re_lage
; var_fakt wiederherstellen
    movlw   0x61            ; Adresse gespeicherte re_lage
    movwf   EEADR
    bsf     EECON1,RD       ; lesen
    movf    EEDAT,W         ; laden
    movwf   var_fakt
    BCF     STATUS,RP0      ; Bank0
; --
    movwf   var_zahl        ; Stellgeschwindigkeit
; Porteinstellungen TRISIO und ANSEL vorbereiten 
    bsf     CMCON0,CM0      ; Comparatoreingnge abschalten
    bsf     CMCON0,CM1
    bsf     CMCON0,CM2
; --
    bsf     GPIO,GP4        ; LED beim Start aus
    BSF     STATUS,RP0      ; Bank1
    movlw   B'00101001'     ; GPIO3 ist Eingang >> MCLR abgeschaltet
                            ; der RESET-Eingang wird am anfang des Listings
                            ; mit "__config _MCLRE_OFF" abgeschaltet !!!
                            ; GPIO4 Ausgang fr LED Programmiermodus
                            ; GPIO2 ist Ausgang -- PWM
                            ; GPIO1 als Ausgang fr Statusanzeige
                            ; im Analogmodus als Eingang (Poti)
    movwf   TRISIO
    movlw   B'00111111'     ; Pull-Up-Wiederstnde ein
    movwf   WPU
    movlw   B'01110000'     ; Pin6 - GP1 als Analogeingang
    movwf   ANSEL           ; ANALOG AUS
    BCF     STATUS,RP0      ; Bank0 
; --
; ----------- alte Einstellung ( vor Stromausfall wiederherstellen --------
    bsf     STATUS,RP0      ; Bank1
    movlw   0x60            ; Adresse gespeicherte lage
    movwf   EEADR
    bsf     EECON1,RD       ; lesen
    movf    EEDAT,W         ; laden letzte Lage in WREG
    bcf     STATUS,RP0      ; Bank0
; --
    movwf   var_time
    movwf   var_tm_ist
    movwf   var_tm_soll
; ----------- Timer0 starten  Vorteiler 128:1 TMR0=115 >> 18ms -----------
; --                variabler Puls 1,0 -2,0ms (in ISR)
    BSF     STATUS,RP0      ; Bank1
    movlw   B'00000110'     ; Vorteiler 128:1
    movwf   OPTION_REG      ; laden OPTION-Register
    BCF     STATUS,RP0      ; Bank0
; --
    movlw   D'115'          ; Zhler  115 << 18ms
    movwf   TMR0            ; Timer0 setzen 
    bcf     INTCON,T0IF     ; Timer0 berlauf ausschalten
    bsf     INTCON,T0IE     ; Interrupt Timer0 einschalten
; -------------- Timer1 vorbereiten --------------------------------------- 
; Timer1 steuert die LED zur anzeige des Programmiermodus
; ". . . ." Einstellung des rechten Anschlages
; "- - - -" Einstaellung des linken Anschlages
; TMR1H = 0x03E konstant
; Vorteiler lang 1:8 0,4s -- kurz 1:4 0,2ms -- Pause 1:2 0,1s
    bcf     T1CON,TMR1ON    ; Timer1 stoppen
    bcf     PIR1,TMR1IF     ; berlauf abschalten
    movlw   0x03E           ; Zeitkonstante fr alle
    movwf   TMR1H           ; Laden TMR1x
    clrf    TMR1L
    bsf     T1CON,T1CKPS0   ; setzen Vorteiler 1:2 (Pause9
    bcf     T1CON,T1CKPS1   
; --
    bsf     STATUS,RP0      ; Bank1
    bsf     PIE1,TMR1IE     ; Interrupt einschalten
    bcf     STATUS,RP0      ; Bank0
; --
    bsf     INTCON,PEIE     ; Interrupt Peripherie(auchn Timer1) freigeben
; --- Initialisierung Timer1+2 fertig << Interrupt freigeben -------------
    bsf     INTCON,GIE      ; Gesamtinterrupt freigeben
; ========================================================================
; ------------------------------------------------------------------------
loop
    bsf     GPIO,GP4        ; LED beim Start aus
; --- testen auf Analogmodus ---------------------------------------------
    btfsc   GPIO,GP0        ; Test auf = 0 ta_links gedrckt
    goto    digital         ; ta_links nicht gedrckt
    btfss   GPIO,GP5        ; Test auf = 1 ta_rechts nicht gedrckt
    goto    analog          ; beide Tasten gedrckt << Analogmodus
; ------------------------------------------------------------------------
; ---- digitalmodus Hauptschleife ----------------------------------------
digital
; Taster abfragen
    btfss   GPIO,0          ; Taster rechts
    goto    ta_rechts
    btfss   GPIO,5          ; Taster links
    goto    ta_links
    btfss   GPIO,3          ; Taster Programmiermodus
    goto    ta_prog     
    GOTO    digital
; rechts stellen GP0 = 0 1ms
ta_rechts
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,0          ; Test ob Taste noch gedrckt
    goto    digital         ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
    movf    re_lage,0       ; 1ms Impuls
    movwf   var_tm_soll
    movwf   ee_dat          ; auf EEPROM Adr 0x60 ablegen
    movlw   0x60
    call    up_eeprom
    goto    digital
; links stellen  GP5 = 0 2ms
ta_links    
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,5          ; Test ob Taste noch gedrckt
    goto    digital         ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
    movf    li_lage,0       ; 2ms Impuls
    movwf   var_tm_soll
    movwf   ee_dat          ; auf EEPROM Adr 0x60 ablegen
    movlw   0x60
    call    up_eeprom
    goto    digital
; ======== ta_prog ============ Programmiermodus GP3 = 0 Taste gedrckt ==
ta_prog
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,3          ; Test ob Taste noch gedrckt
    goto    digital         ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
    bsf     T1CON,TMR1ON    ; Timer1 fr das blinken einschalten
; Taste ist gedrckt x_stat4:3  rcksetzen
    bcf     x_stat,1        ; Tastenstatus auf aus
    bcf     x_stat,3        ; x_stat Bit3 = 0
    bcf     x_stat,4        ; x_stat Bit4 = 0
; warten auf Taste losgelassen
prog_ta_w
    btfss   GPIO,GP3        ; Taste PROG
    goto    prog_ta_w       ; noch gedrckt
; --- Test ob Geschwindikeitseinstellung ---------------------------------
    movf    x_stat,W        ; Status nach W
    andlw   B'00011000'     ; Bit 4:3 erhalten Rest ausblenden
    xorlw   B'00000000'     ; testen auf 00 also Z-Flag
    btfsc   STATUS,Z        ; wenn Z-Flag nicht gesetzt
    goto    prog_ta_z1      ; 
; --- Test ob zum rechten Anschlag aus dem x_status Bit4:3 = 01 ----------
    movf    x_stat,W        ; Status nach W
    andlw   B'00011000'     ; Maske Bit4:3
    xorlw   B'00001000'     ; testen auf 01 = Z-Flag
    btfss   STATUS,Z        ; wenn Z dann war Bit4:3 = 00 re Anschlag stellen
    goto    prog_ta_za      ; wenn nicht testen auf Bit4:3 = 01
; Einstellung Timer1 auf rechten Anschlag setzen
    bsf     T1CON,T1CKPS1   ; Vorteiler auf kurze LED-Impulse
    bcf     T1CON,T1CKPS0   ; 1:4                       ;
    goto    prog_ta_z1
; --- Test ob zum linken Anschlag aus dem x_status Bit4:3 = 10 -----------
prog_ta_za                  ; 
    movf    x_stat,W
    andlw   B'00011000'     ; Maske Bit4:3
    xorlw   B'00010000'     ; testen auf 10 = Z-Flag
    btfss   STATUS,Z        ; wenn Z dann war Bit4:3 = 10 li Anschlag stellen
    goto    prog_ta_end     ; dann war vorher linker anschlag
; Einstellung  Timer1 auf linken Anschlag
    bsf     T1CON,T1CKPS1   ; Vorteiler auf lange LED-Impulse
    bsf     T1CON,T1CKPS0   ; 1:8   
; ------------ 
; ----- warten auf gedrckte Taste t_prog, t_rechts, t_links -------------
prog_ta_z1
    btfss   GPIO,GP3        ; Taste t_prog nicht gedrckt
    goto    prog_ta_um      ; Rcksprung weil t_prog gedrckt - zum nchsten Modus
    btfss   GPIO,GP0        ; Taste t_rechts nicht gedrckt
    goto    prog_ta_ent0    ; zum Entprellen
prog_ta_ent5
    btfsc   GPIO,GP5        ; Taste t_links gedrckt    
    goto    prog_ta_z1
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,5          ; Test ob Taste noch gedrckt
    goto    prog_ta_z1      ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
; -------- testen auf Geschwindikeitseinstellung starten  ----------------
prog_ta_z2
    movf    x_stat,W        ; Status nach W
    andlw   B'00011000'     ; Bit 4:3 erhalten Rest ausblenden
    xorlw   B'00000000'     ; testen auf 00 also Z-Flag
    btfsc   STATUS,Z        ; wenn Z-Flag nicht gesetzt
    goto    prog_ta_ges     ; Einstellung der Bewegungsgeschwindigkeit
; ----------- rechten und linken Anschlag bearbeiten ---------------------
prog_ta_z3
    btfss   x_stat,3        ; Test ob "rechter Anschlag"
    goto    prog_ta_li1
; ------------------ rechten Anschlagstellen bearbeiten ------------------
    movf    re_lage,W
    movwf   var_tm_soll
    btfsc   GPIO,GP0        ; Taste t_rechts gedrckt
    goto    prog_ta_re2
    movlw   0xDC            ; obere Grenze 0xDC
    xorwf   re_lage,W
    btfss   STATUS,Z        ; wenn obere Grenze errecht nicht weiter
    incf    re_lage,1       ; um 1 erhhen
    goto    prog_ta_re3
prog_ta_re2
    btfsc   GPIO,GP5        ; Taste t_links gedrckt
    goto    prog_ta_lire1
    movlw   0x6C            ; untere Grenze ist 0x6C
    xorwf   re_lage,W
    btfss   STATUS,Z        ; wenn untere Grenze erreicht nicht weiter
    decf    re_lage,1       ; um 1 verringern
prog_ta_re3
    movf    re_lage,0
    movwf   var_tm_soll
; warten auf beide Tasten losgelassen
prog_ta_lire1
    btfss   GPIO,GP5        ; warten bis t_links aus
    goto    prog_ta_lire1
prog_ta_lire2
    btfss   GPIO,GP0            ; warten bis t_rechts aus
    goto    prog_ta_lire2
    goto    prog_ta_z1      ; zurck zur Schleife
; ----------------------- linken Anschlag bearbeiten --------------------
prog_ta_li1
    btfsc   GPIO,GP0        ; Taste t_rechts gedrckt
    goto    prog_ta_li2
    movlw   0xDC            ; obere Grenze 0xDC
    xorwf   li_lage,W
    btfss   STATUS,Z        ; wenn obere Grenze errecht nicht weiter
    incf    li_lage,1       ; um 1 erhhen
    goto    prog_ta_li3
prog_ta_li2
    btfsc   GPIO,GP5        ; Taste t_links gedrckt
    goto    prog_ta_lire1
    movlw   0x6C            ; untere Grenze ist 0x6C
    xorwf   li_lage,W
    btfss   STATUS,Z        ; wenn untere Grenze erreicht nicht weiter
    decf    li_lage,1       ; um 1 verringert
prog_ta_li3
    movf    li_lage,0
    movwf   var_tm_soll
    goto    prog_ta_lire1
; -------------- Endebehandlung Programmiermodus -------------------------
prog_ta_end
    ; sichern von var_tm_soll auf eeprom 
    bcf     T1CON,TMR1ON    ; Timer1 stoppen
; ------------------------------------------------------------------------
; li_lage und re_lage im EEPROM sichern
; in Zelle 70 li_lage sichern
    movf    li_lage,W
    movwf   ee_dat
    movlw   0x70
    call    up_eeprom
; in Zelle 71 re_lage sichern
    movf    re_lage,W
    movwf   ee_dat
    movlw   0x71
    call    up_eeprom
; in Zelle 61 var_fakt sichern
    movf    var_fakt,W
    movwf   var_zahl        ; Zhler fr Stellgeschwindigkeit
    movwf   ee_dat
    movlw   0x61
    call    up_eeprom
; Ende des Programmiermodus
    bsf     GPIO,GP4        ; LED ausschalten
    goto    digital
;-------------------------------------------------------------------------
; ---------------- Stellgeschwindigkeit bearbeiten -----------------------
prog_ta_ges 
    btfss   GPIO,0          ; Taste T1
    goto    prog_ta_ge1
    btfsc   GPIO,5          ; Taste T2
    goto    prog_ta_ge2     ; Zyclus Ende
    btfsc   var_fakt,5      ; Test auf var_fakt = 32  maximum
    goto    prog_ta_ge2     ; Zyclus Ende
    movf    var_fakt,F      ; var_fakt auf 0 testen 
    btfss   STATUS,Z
    goto    prog_ta_ge3     ; zur Linksverschiebung
    incf    var_fakt,F      ; weil var_fakt = 0 << auf 1 setzen
    goto    prog_ta_ge2
prog_ta_ge1
    movf    var_fakt,F      ; Test ob 0 << kopieren auf sich selbst  
    btfsc   STATUS,Z        ; wenn Z-Flag = 1 << F-Register = 0
    goto    prog_ta_ge2     ; Zyclus Ende
    bcf     STATUS,C        ; vorsorglich C-Flag auf 0
    rrf     var_fakt,F      ; rechts verschieben << halbieren
; warten auf Taten losgelessen
prog_ta_ge2 
    btfss   GPIO,5
    goto    $-1
    btfss   GPIO,0
    goto    $-1
    goto    prog_ta_z1      ; warten auf nchsten Tastendruck
; Stellzeit verdoppeln
prog_ta_ge3
    bcf     STATUS,C        ; vorsichtshalber C-Flag lschen
    rlf     var_fakt,F      ; links verschieben = verdoppeln
    goto    prog_ta_ge2     ; warten auf Tasten losgelassen
; ----------------- auf nchsten Programmiermodus schalten
prog_ta_ent0
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,0          ; Test ob Taste noch gedrckt
    goto    prog_ta_z1      ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
    goto    prog_ta_z2
prog_ta_um
    ; Entpellzeit Anfang 60ms warten
    movlw   0x03            ; 3 Zyclen a 20ms einstellen
    movwf   z_entprell      ; der Zhler wird in der ISR PWM herunter gezhlt
    movf    z_entprell,F    ; Test auf 0
    btfss   STATUS,Z
    goto    $-2             ; Rcksprung bis Zhler 0             
    btfsc   GPIO,3          ; Test ob Taste noch gedrckt
    goto    prog_ta_z1         ; wenn nicht dann Rcksprung
    ; Entprellzeit Ende
    movlw   B'00001000'
    addwf   x_stat,F
    goto    prog_ta_w
; ========================================================================
; --------- Analogmodus
analog
; ------------ ANALOG vorbereiten ----------------------------------------
; Analogeingang GP1 Pin6 = AN1
; Ergebnis in Bank1 in ADRESH 8Bit die 2 LSB fallen 
; damit erhlt man 256 statt 1024 Stufen
; fr die Einstellung des var_time (Impuls fr Servo) 
; sind ca. 110 Schritte mglich
; --
    BSF     STATUS,RP0      ; Bank1
    movlw   B'00101011' 
    movwf   TRISIO
    movlw   B'01110010'     ; Interner Oszilator und AN1 ausgewhlt
    movwf   ANSEL
    BCF     STATUS,RP0
; --
    movlw   B'00000100'     ; Ergebnis linksbndig -- Kanal1 -- AD aus
    movwf   ADCON0          ; ADC aus -- Conversion gestoppt
; ------------------------------------------------------------------------
; ---- warten bis keine Taste gedrckt ist
    bsf     GPIO,GP4        ; LED abschalten
    btfss   GPIO,GP0        ; Test auf = 0 ta_links gedrckt
    goto    analog          ; ta_links nicht gedrckt
    btfss   GPIO,GP5        ; Test auf = 1 ta_rechts nicht gedrckt
    goto    analog          ; beide Tasten gedrckt << Analogmodus          
; 
    bsf     ADCON0,ADON     ; Einschalten A/D-Modul 
    bsf     x_stat,6        ; Anzeige Analogmodus
ana_z1
    btfsc   ADCON0,GO       ; Test ob Messung fertig
    goto    ana_z1
; -------- Messwert nach var_ta_soll (Umrechnung) ---------------------------
    movf    ADRESH,W        ; Messwert in den PUFFER stellen
    movwf   ana_pu
    rrf     ana_pu,F        ; Messwert durch 2 geteilt und nach WREG
    bcf     ana_pu,7        ; oberstes Bit lschen

    movlw   0x08            ; Subtraktion 8 Stellbereich unten abschneiden
    subwf   ana_pu,F
    btfss   ana_pu,7        ; wenn Bit7 = 1 dann war Puffer <= 8 (0x08)
    goto    ana_te2         ; wen also >=10 dann zu ana_te2

    clrf    ana_pu          ; der Puffer war <= 10 also auf 0 lschen
    goto    ana_z2
ana_te2
    movlw   0x89            ; 0x89 (137) + Puffer >= 255 ergibt C-Flag      
    addwf   ana_pu,W        ; addieren 137
    btfss   STATUS,C        ; wenn C-Flag gesetzt war Puffer zu gross
    goto    ana_z2          ; nein Puffer war kleiner 118 also OK
    movlw   0x6E            ; maximalwert setzen
    movwf   ana_pu
ana_z2
    movlw   0x6D            ; rechter Anschlag
    addwf   ana_pu,W        ; + gemessener und bereinigter Wert
    movwf   var_tm_soll

    goto ana_z1
; ========================================================================
; --------- UP EEPROG beschreiben eines EEPROM-Speicherplatzes -----------
; Speicheradresse im WREG
; Datebyte in ee_dat 
up_eeprom
    bsf     STATUS,RP0      ; Bank1
    movwf   EEADR
    movf    ee_dat,W
    movwf   EEDAT
    bsf     EECON1,WREN     ; jetzt schreiben erlauben
    bcf     INTCON,GIE      ; Interrupt verbieten
    btfsc   INTCON,GIE      ; warten auf GIE abgeschaltet
    goto    $-1
    ; anfang !!! vorgeschriebene Codefolge
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1,WR       ; schreiben starten
    ; ende !!! vorgeschriebene Codefolge
    bsf     INTCON,GIE
    bcf     STATUS,RP0      ; Bank0
    btfss   PIR1,EEIF       ; Test ob schreiben zu fertig
    goto    $-1
    bcf     PIR1,EEIF       ; Fertigmeldung wieder abschalten
    return
; ========================================================================
    END                       ; directive 'end of program'
; ************************************************************************
